<?php

/**
 * This hook allows modules declare what widgets they make available. It
 * can be charts, tables or whatever information to be added on store overview
 * page. Widgets are also used for blocks and panels.
 *
 * @return
 *   Array, each widgets' attributes must include
 *   - title: the widget title
 *   - description: information about the widget
 *   - callback: function which returns the content
 *   And optionally include:
 *   - weight: for default ordering
 *   - perm: access control string, (defaults to administer store)
 *
 * @todo get widgets working with panels 2.
 */
function hook_ec_widgets() {
  return array(
    'orders_basic' => array(
      'title'       => t('Orders'),
      'description' => t('Basic orders information.'),
      'callback'    => '_store_widget_orders',
      'weight'      => -10,
      'perm'        => 'administer store',
    ),
  );
}

/**
 * Return e-Commerce permissions to ec_common - this allows all e-Commerce
 * permissions to appear in the same place.
 *
 * See hook_perm() for usage.
 */
function hook_ec_perm() {
  return array('manage subproducts');
}

/**
 * Handle store transaction actions.
 *
 * @param &$txn
 *   A transaction object, passed by reference.
 * @param $op
 *   A string containing the name of the ec_transactionapi operation.
 *   Possible values:
 *   - "insert":
 *       A new transaction is being created.
 *       Called by e.g. store_transaction_save(), with $txn an object containing edit fields
 *       Standard records in ec_transaction etc. have already been inserted. No return value.
 *   - "load":
 *       A transaction is being loaded from the database.
 *       Called by e.g. store_transaction_load(), with $txn containing the transaction loaded so far.
 *   - "update":
 *       A transaction is being updated.
 *       Called by e.g. store_transaction_save(), with $txn an object containing edit fields
 *       Standard records in ec_transaction etc. have already been updated. No return value.
 *   - "delete":
 *       A transaction is being deleted.
 *       The parameter $txn contains all the transaction information,
 *       since the standard records in ec_transaction etc. have already been deleted. No return value.
 *   - "validate":
 *       Transaction $txn is being inserted. Argument $a3 contains the section to validate.
 *       Errors should be set with form_set_error(). No return value.
 *       Called by e.g. store_transaction_validate(), from e.g.
 *       Menu: ?q=admin/store with $_POST['op'] == [ t('Update transaction') or t('Create new transaction') ]
 *       on submission of a form to create a new transaction or update an existing transaction.
 * @param $a3
 *   Additional argument, depending on $op.
 *   - For "validate", this is the section to validate, in upper case.
 *     Possible values include:
 *     - 'OVERVIEW':
 *     - 'ADDRESSES':
 *     - 'ITEMS':
 *     - 'ALL' (default): Validate all sections.
 * @param $a4
 *   Additional argument, depending on $op. (Not currently used?)
 * @return
 *   The returned value of the invoked hooks, depending on $op.
 *   - The "load" operation should return an associative array of "property => value" pairs
 *     to be merged in to the loaded transaction object.
 *     (Note: do not return an object, as you would with hook_load(),
 *     since store_invoke_ec_transactionapi() will ignore it)
 */
function hook_ec_transactionapi(&$txn, $op, $a3 = NULL, $a4 = NULL) {
  switch ($op) {
    case 'insert':
      // Empty the cart for this user.
      // Note that user must be logged in at this point.
      cart_empty($txn->uid);
  }
}


/**
 * hook_product_ec_checkout_validate_item().
 *
 * When ever the cart is updated this is called to make sure that the change
 * is valid, and allowed by different products.
 *
 * @param $node
 *  The item that is being added to the cart.
 *
 * @param $type
 *  Specifies the type of transaction that this product is being validated 
 *  for.
 *
 * @param $data
 *  If there is any data that is attached to an item which has been put into
 *  the cart then this is added here.
 *
 * @return
 *  This returns TRUE or FALSE. If FALSE is returned then the item will not
 *  be added to the cart. It will be the responsibility of the module to
 *  display any error messages. If a new node id is returned, then
 *  this will be used instead if the node id which was passed.
 */
function hook_product_ec_checkout_validate_item($node, $type, $qty, $data) {
}

/**
 * ecommerceapi for "on payment completion".
 *
 * Example taken from ec_useracc.
 *
 * @param &$txn
 *   A transaction object, passed by reference.
 * @param $op
 *   A string containing the name of the ecommerceapi operation.
 *   Possible values:
 *   - "on payment completion":
 *     Allows modules to hook into the payment completion event.
 */
function hook_ecommerceapi(&$txn, $op) {
  global $user;

  switch ($op) {
    case 'on payment completion':
      if ($user->uid == $txn->uid) {
        return;
      }

      // Go through each of the products purchased.
      for ($i = 0; $i < count($txn['items']); $i++) {
        $node =& $txn['items'][$i]; // easy referencing

        // ec_useracc_load($node);

        if ($node->useracc['create']) {
          if (empty($txn->uid)) {
            ec_useracc_create($txn, $node);
          } else {
            ec_useracc_unblock($txn->uid);
          }
          break;
        }
      }

      // ec_roles_purchase.
      break;
  }
}

/**
 * Determine shipping methods implemented by a module for shipcalc.module.
 *
 * For usage example, see shipcalc.module and the .inc files in its partners folder.
 *
 * @return
 *  An array of shipping methods.
 *
 */
function hook_shipping_methods() {
  $methods['ups'] = array(
    '#title' => t('UPS'),
    '#description' => t('United Parcel Service of America, Inc.')
  );
  $methods['ups']['1DM'] = array(
    '#title' => t('Next Day Air Early AM'),
  );
  // <snip />

  return $methods;
}

/**
 * Add email data to be used within e-Commerce modules.
 *
 * This hook allows us to add e-Commerce emails. Also, this can be used
 * in conjunction with store_email_form method that renders email
 * customization and preview forms.
 *
 * @param $messageid
 *  A string that identifies the message. We need to check if this match
 *  the messages we're additing, using an if() or switch() statement.
 *
 * @return
 *  Return an array that contains the message data (ie. body, subject and variables).
 */
function hook_store_email_text($messageid) {
  if ($messageid == 'shipping_notification') {
    return array(
      'subject' => t("Your %site order has shipped (#%txnid)"),
      'body' => t("Hello %first_name,\n\nWe have shipped the following item(s) from Order #%txnid, received  %order_date.\n\nItems(s) Shipped:\n%items\n%shipping_to\nQuestions about your order? Please contact us at %email.\n\nThanks for shopping at %site.  We hope to hear from you again real soon!\n\n%uri"),
      'variables' => array('%order_date', '%txnid', '%billing_name', '%first_name', '%user_data', '%billing_to', '%shipping_to', '%items', '%email', '%site', '%uri', '%uri_brief', '%date')
    );
  }
}

/**
 * Alter one or many e-Commerce emails data.
 *
 * With this hook we can alter e-Commerce emails, the parameters goes as
 * reference, so we can alter either the message data (body, subject and
 * variables) and the message variables before sending the email.
 *
 * @param $messageid
 *  A string that identifies the message. We need to check if this match
 *  the messages we're altering, using an if() or switch() statement.
 *
 * @param $message
 *  An array that contains the message data (ie. body, subject and variables).
 *
 * @param $variables
 *  An indexed array with the variables available for this message and their
 *  respective values.
 *
 * @return
 *  This hook has no return value.
 */
function hook_store_email_alter($messageid, &$message, &$variables) {
  if ($messageid == 'shipping_notification') {
    $message->subject = t("Your %site order has shipped");
    $variables['%site'] = 'drupal.org';
  }
}

/**
 * Send a e-Commerce email using other mechanism than the default.
 *
 * This hook abstracts the sending email process. This can be used
 * to send a html/mime email or to put the message in a queue.
 *
 * @param $from
 *  The email address to be used as sender.
 *
 * @param $to
 *  The recipient where to send this email.
 *
 * @param $subject
 *  The subject of this email message.
 *
 * @param $body
 *  The parsed message body string.
 *
 * @param $headers
 *  Optional email headers to be added to the email.
 *
 * @return
 *  This should return TRUE indicating that the message was sucessful delivered,
 *  FALSE if an error occurred while delivering the message and none if this
 *  implementation doesn't override the email delivering process.
 */
function hook_store_email_send($from, $to, $subject, $body, $headers) {
  return mail($to, $subject, $body, $headers);
}

/**
 * Implementation of hook_recurringapi().
 *
 * @param $op Event the hook is reacting to
 *    'on expiry': called when a product has expired
 *    'on purchase': called when a product is purchased
 *    'get previous purchase': called so modules can set the
 *          $node->recurring['prevpurchase'] value and override the system default
 *          WARNING: Do this with great caution and ensure the format of the value
 *          matches that of the default or you'll break the system!
 *    'expiry schedule changed': called when a product schedule has changed
 *    'cron report': called so modules can implement their own reporting code utilizing
 *          values in $GLOBALS['expirations']
 * @param $obj Reference to data specific to the event
 *    'on expiry': Node of the expired product. In addition to the node fields
 *          a $node->expired_schedule member is set to be the row from the
 *          ec_recurring_expiration table. NOTE: you must be careful to check the status
 *          field of this row before performing operations. The product may be renewed
 *          (i.e have a value of ECRECURRING_STATUS_RENEWED) and may not need any
 *          additional processing other than setting the status to expired. The system
 *          does that automatically.
 *    'on purchase': Transaction
 *    'get previous purchase': Node for the current purchase
 *    'expiry schedule changed': Product node with $node->oldsid set to the previous
 *          schedule ID
 *    'cron report': $GLOBALS['expirations'] containing values collected during
 *          expiration processing
 * @param $val1: First extra value specific to the event
 *    'on expiry': The expiry timestamp
 *    'on purchase': The node ID for the purchased product
 *    'get previous purchase': User ID of the customer
 *    'expiry schedule changed': NULL
 *    'cron report': NULL
 * @param $val2: Second extra value specific to the event
 *    'on expiry': NULL
 *    'on purchase': TRUE if this is a renewal of the product
 *    'get previous purchase': NULL
 *    'expiry schedule changed': NULL
 *    'cron report': NULL
 */
function hook_recurringapi($op, &$obj, $val1 = NULL, $val2 = NULL) {
  switch ($op) {
    case 'on expiry':
      ec_recurring_on_expiry($obj, $val1);
      break;
    case 'on purchase':
      break;
    case 'get previous purchase':
      break;
    case 'expiry schedule changed':
      break;
    case 'cron report':
      // This is intended for use in the cron mail.
      print t("Processed %nprocessed expired products\n%nauto automated payments processed\n\n", array('%nprocessed' => $obj['products_processed'], '%nauto' => $obj['autopay']));
      print t("Processed %nprocessed expired reminders\nSent %nsent reminders\n%nfailed reminders failed\n\n", array('%nprocessed' => $obj['reminders_processed'], '%nsent' => $obj['reminders_sent'], '%nfailed' => $obj['reminders_failed']));
      break;
  }
}

/**
 * Implementation of hook_invoiceapi().
 *
 * @param $op Event the hook is reacting to
 *
 * @param $obj Reference to data specific to the event
 *
 * @param $val1: First extra value specific to the event
 *
 * @param $val2: Second extra value specific to the event
 *
 */
function hook_invoiceapi($op, &$obj, $val1 = NULL, $val2 = NULL) {
  switch ($op) {
    case '':
      break;
    case '':
      break;
    case '':
      break;
    case '':
      break;
    case '':
      break;
    case '':
      break;
  }
}


/**
 * Define the type of Mail that can be sent.
 *
 * Implement this hook to create additional mail types that can be
 * managed by the user and called by your code.
 */
function hook_mail_types() {
  return array(
    ECMAIL_TYPE_CUSTOMER_INVOICE => t('Customer invoice'),
    ECMAIL_TYPE_PROCESSING_ERROR => t('Processing error notification'),
    ECMAIL_TYPE_CANCEL_TXN => t('Transaction cancelled notice'),
    ECMAIL_TYPE_ASK_CUSTOMER => t('Query to customer'),
  );
}

/**
 * FIXME: Add Description that means something
 * ^^ Sorry, I can't tell what the function does!
 */
function hook_mail_reset($type) {
  $mids = array();
  $mid = NULL;
  $var = NULL;

  switch ($type) {
    case ECMAIL_TYPE_CUSTOMER_INVOICE:
      $defsub = t('Your %site order');
      $defbody = t("Dear %billing_name,\n\nThanks for your recent purchase from %site.  This message includes important information about your order. Please take a moment to read it closely, and be sure to save a copy for future reference.\n\n********************************\nBilling and Shipping Information\n********************************\n%user_data\n********************************\nOrder Details\n********************************\n%items\nQuestions about your order? Please contact us at %email\n\n********************************\nShipping Instructions\n********************************\nWe will notify you by email as soon as your order ships.\n\nThanks for shopping at %site.  We hope to hear from you again real soon!\n\n%uri");
      $mid = ec_mail_import_old_mail(t('Default customer invoice'), $type, 'store_email_customer_invoice_subject', $defsub, 'store_email_customer_invoice_body', $defbody);
      $var = MAILVAR_CUSTOMER_INVOICE;
      break;
    case ECMAIL_TYPE_PROCESSING_ERROR:
      $defsub = t('Purchase problem, %site');
      $defbody = t("Dear Customer\n\nIt seems there was a problem while processing your order (%txnid). Please contact us at %email for further details.\n\nRegards,\n%site team\n%uri");
      $mid = ec_mail_import_old_mail(t('Default processing error'), $type, 'store_email_processing_error_subject', $defsub, 'store_email_processing_error_body', $defbody);
      $var = MAILVAR_PROCESSING_ERROR;
      break;
    case ECMAIL_TYPE_CANCEL_TXN:
      $defsub = t('Your %site order has been canceled');
      $defbody = t("Dear %billing_name,\n\nYour order (%txnid) has been canceled. Please contact us at %email for further details.\n\nRegards,\n%site team\n%uri");
      $mid = ec_mail_import_old_mail(t('Default order cancellation notice'), $type, 'store_email_cancel_transaction_subject', $defsub, 'store_email_cancel_transaction_body', $defbody);
      $var = MAILVAR_CANCEL_TXN;
      break;
    case ECMAIL_TYPE_ASK_CUSTOMER:
      $defsub = t('Questions regarding your order from %site');
      $defbody = t("Dear %billing_name,\n\nWe have some questions regarding your order from %site.\n\nRegards,\n%site team\n%uri");
      $mid = ec_mail_import_old_mail(t('Default query to customer'), $type, 'store_email_ask_customer_template_subject', $defsub, 'store_email_ask_customer_template_body', $defbody);
      $var = MAILVAR_ASK_CUSTOMER;
      break;
  }

  if (!empty($var)) {
    ec_mail_variable_change($var, $mid, TRUE);
  }

  if (!empty($mid)) {
    $mids[] = $mid;
  }

  return $mids;
}

/**
 * Process changes to the system when the transaction workflow changes
 *
 * Whenever the workflow of a transaction changes the system will invoke
 * this hook to allow the system to make changes outside of e-Commerce such
 * as create transactions for the General Ledger interface.
 *
 * @param $txn
 *  The transaction object after the transaction has been saved.
 *
 * @param $orig_txn
 *  The transaction object before the transaction was saved.
 */
function hook_ec_txn_workflow($txn, $orig_txn) {
  // TODO - Create example
}

/**
 * @} End of "ingroup hooks".
 */

/**
 * @} End of "defgroup e-Commerce"
 */

