<?php

/**
 * Define one or more product types.
 *
 * This hook is used by product modules. This hook is required to define one
 * or more product types. It is called to determine the names and
 * attributes of a module's product types.
 *
 * @return
 * An array of information is returned which defines a module based product.
 * <ul>
 * <li>"name": The human readable name of the product type. Required.</li>
 * <li>"module": This string tells e-Commerce how to map the hook functions
 * to modules.</li>
 * <li>"description": a brief description of the product type. Required</li>
 * </ul>
 */
function hook_product_info() {
  return array(
    'product' => array(
      'name' => t('Fancy product'),
      'module' => 'fancy',
      'description' => t('A fantastic product which does neat things.'),
    ),
  );
}

/**
 * hook_product_attributes().
 *
 * @param $node
 *  This is the node of the product which is being inspected. The attributes
 *  for this product are based upon the product node which is past in.
 *
 * @param $attribute
 *  Optionally pass an attribute by name in order to verify the existence of that
 *  attribute.
 *
 * @return
 *  Returned is an indexed list of the attributes which pertain to this
 *  product. These can be either TRUE or FALSE. If the attribute that is
 *  being searched for does not exist, then it is FALSE.
 */
function hook_product_attributes($node, $attribute = NULL) {
  return array('no_stock' => TRUE, 'no_quantity' => TRUE, 'no_discounts' => TRUE);
}

/**
 * hook_product_transaction()
 */
function hook_product_transaction($node, $op) {
}

/**
 * hook_product_price_adjusts()
 */
function hook_product_price_adjusts($node, $old_price, $type, $txn) {
  if (!((float)$node->price)) {
    return $node->gc_price;
  }
}

/**
 * hook_product_subproduct_types()
 */
function hook_product_subproduct_types($node) {
  return array('sandwich');
}

/**
 * hook_product_validate()
 */
function hook_product_validate($node, $form) {
}

/**
 * hook_product_load()
 */
function hook_product_load($node) {
}

/**
 * hook_product_insert()
 */
function hook_product_insert($node) {
}

/**
 * hook_product_update()
 */
function hook_product_update($node) {
}

/**
 * hook_product_delete()
 */
function hook_product_delete($node, $revision_only) {
}


/**
 * Create a special for products.
 *
 * If you have products which have a special/discount site wide before the
 * checkout process begins. This can be used for things like timed specials,
 * or discounts based upon the users roles.
 *
 * @param $node
 *  This is the node for the product that we need to know the specials
 *  for.
 *
 * @param $specials
 *  Because this hook can be used by multiple modules at the same time it
 *  is possible that if a special has already been placed on an order the
 *  it may affect the current special.
 *
 * @param $txn
 *  If the special is going to be added to a transaction such as a cart
 *  item, or an invoice then the $txn will be passed so that it can be used to
 *  determine the correct user or other information.
 *
 * @return
 *  This will return an array which has a list of the specials by the key
 *  which will appear in misc, and the actual value of the discount. This
 *  will usually be a negative value.
 *
 */
function hook_product_specials($node, $specials, $txn) {
  return array('special' => -5);
}

/**
 * Alter the node form, only if it is a product.
 *
 * The benefit of using this hook over a normal form alter are:
 *  - product.module is better placed to determine if the node is a product
 *  - your code will run after product.module alterations
 *  - your form items will appear in a product fieldset.
 *  - if a site developer wants to alter the entire productized node form,
 *    then they need only make sure their form_alter runs after product form.
 *  - if you choose to return form items (rather than just alter the form array)
 *    then they will be placed in a product fieldset (so you don't need to worry
 *    about the structure of the form).
 *
 * @param $pform
 *  A product form that is being built. This should be passed by reference.
 *
 * @param $node_form
 *  The actual node form, prior to product alteration, that can be altered as well.
 *  $node_form['#node'] contains existing values of the node.
 *
 * @return n/a
 */
function hook_product_form_alter(&$pform, &$node_form) {
  // Example from ec_useracc_product_form_alter()
  if (product_feature_exists($node_form['#node'], 'ec_useracc') && product_feature_exists($node_form['#node'], 'ec_recurring')) {
    $pform['useracc'] = array(
      '#type' => 'fieldset',
      '#title' => t('User account provision'),
      '#weight' => -14,
      '#collapsible' => false,
      '#collapsed' => false,
    );
    $pform['useracc']['useracc_block'] = array(
      '#type' => 'checkbox',
      '#title' => t('Block the user\'s account when this product expires.'),
      '#default_value' => ($disabled ? false : $node_form['#node']->useracc['block']),
      '#description' => $desc
    );
  }
}