<?php

/**
 * @file ec_common.manage.inc
 *   This module contains generic functions for ecommerce which are typically available
 *   to the user with 'store admin manage' permissions.
 */


// -------------------------------------------------------------------------
// e-Commerce overview

/**
 * Generates statistical data of the store with links to view and the details
 * of transactions.
 */
function ec_common_manage_overview() {
  $widgets_list = array();

  // Check what dashboards the user can and want to see
  if ($widgets = module_invoke_all('ec_widgets')) {
    usort($widgets, 'ec_sort');
    foreach ($widgets as $code => $widget) {
      if (ec_common_overview_access($widget['title'])) {
        $widgets_list[$code] = $widget;
      }
    }
  }

  return theme('ec_common_manage_overview', $widgets_list);
}

/**
 * Theme the Store overview page
 *
 * @ingroup themable
 * @todo Use the block admin functions
 */
function theme_ec_common_manage_overview($widgets) {
  $output = '';

  if (empty($widgets)) {
    $output = t('There is no info.');
  }
  else {
    // Add the CSS
    drupal_add_css(drupal_get_path('module', 'store') .'/store.css');

    // Create the two column blocks
    $stripe = 0;
    $container = array('right' => '', 'left' => '');
    while (list($code, $widget) = each($widgets)) {
      $position = $stripe++ % 2 ? 'right' : 'left';
      $container[$position] .= theme('admin_block', array_merge($widget,
        array('content' => call_user_func($widget['callback'], $widget))));
    }
    foreach ($container as $id => $data) {
      $output .= '<div class="'. $id .' clear-block">'. $data .'</div>';
    }
  }

  return $output;
}

// -------------------------------------------------------------------------
// Generic query builder class.

/**
 * e-Commerce query object
 */
class _ec_query {

  var $fields = array();
  var $tables = array();
  var $where = array();
  var $group = array();
  var $having = array();
  var $order = array();
  var $distinct = false;

  function add_select($field) {
    $this->fields[] = $field;
  }

  function ensure_table($table, $alias, $type = null, $join = null) {
    if (!isset($this->tables[$alias])) {
      $this->tables[$alias] = array(
        'table' => $table,
        'alias' => $alias,
        'type' => $type,
        'join' => $join
      );
    }
  }

  function add_where($where) {
    $this->where[] = $where;
  }

  function add_group($group) {
    $this->group[] = $group;
  }

  function add_having($having) {
    $this->having[] = $having;
  }

  function add_order($order) {
    $order = trim($order);
    if (!empty($order)) {
      $this->order[] = $order;
    }
  }

  function is_distinct() {
    $this->distinct = true;
  }

  function query($count = false) {
    $this->_prepare_query();

    $sql = 'SELECT ';
    $fields = $this->fields;
    if ($this->distinct) {
      $fields[0] = 'DISTINCT '. $fields[0];
    }
    if ($count) {
      $fields = array("COUNT({$fields[0]})");
    }
    $sql .= implode(',', $fields);
    $sql .= ' FROM '. implode(' ', $this->from);
    if (!empty($this->where)) {
      $sql .= ' WHERE '. implode(' AND ', $this->where);
    }
    if (!empty($this->group) && !$count) {
      $sql .= ' GROUP BY '. implode(',', $this->group);
    }
    if (!empty($this->having) && !$count) {
      $sql .= ' HAVING '. implode(' AND ', $this->having);
    }
    if (!empty($this->order)) {
      $sql .= ' ORDER BY '. implode(',', $this->order);
    }
    return $sql;
  }

  function get_tables() {
    foreach ($this->tables as $table) {
      $tables[] = $table['table'];
    }
    return array_unique($tables);
  }

  function _prepare_query() {
    $this->fields = array_unique($this->fields);
    $this->where = array_unique($this->where);
    $this->group = array_unique($this->group);
    $this->order = array_unique($this->order);

    $this->from = array();
    foreach ($this->tables as $table) {
      $from = !empty($table['type']) ? "{$table['type']} JOIN " : '';
      $from .= "{{$table['table']}} {$table['alias']}";
      $from .= !empty($table['join']) ? " ON {$table['join']}" : '';
      $this->from[] = $from;
    }
  }
}

// -------------------------------------------------------------------------
// Generic search form.

// TODO: make this form a bit more flexible, allow caller to choose which
//       fields are displayed. Or maybe it already is! Hard to tell.

/**
 * Common search interface system
 */
function ec_search($search_id) {
  $excl = array_merge(array('q', 'op', 'form_token', 'form_id'), array_keys($_COOKIE));
  $query = ec_array_exclude($_REQUEST, $excl, array(''));

  if (is_array($query)) {
    $query = array_map(create_function('$a', 'return (!is_array($a) && strpos(",", $a) ? explode(",", $a) : $a);'), $query);
  }
  else {
    $query = array();
  }
  return drupal_get_form('ec_search_form', $search_id, $query);
}

/**
 * Called by store_transactions_search_form()
 */
function ec_search_form($search_id, $query = NULL) {
  drupal_add_css(drupal_get_path('module', 'store') .'/store.css', 'module');

  $form = array();
  $form['search_id'] = array(
    '#type' => 'value',
    '#value' => $search_id,
  );
  $form['#redirect'] = false;
  $form['#action'] = url($_REQUEST['q']);
  $form['basic'] = array(
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>'
  );
  $form['basic']['text'] = array(
    '#type' => 'textfield',
    '#title' => t('Search for'),
    '#default_value' => empty($query['text']) ? '' : $query['text'],
    '#size' => 60,
    '#maxlength' => 255
  );
  $form['basic']['submit'] = array('#type' => 'submit', '#value' => t('Search'));
  $form['advanced'] =  array(
    '#type' => 'fieldset',
    '#title' => t('Advanced search'),
    '#description' => '<p>'. t('The textfields in this form recognize special operators like greater than (&gt;), less than (&lt;), comma separated values and equals to, which is also the default option if none is specified. This allows you to easily find ranges and multiple values and works with IDs and dates.') .'</p>',
    '#collapsible' => true,
    '#collapsed' => true,
    '#attributes' => array('class' => 'search-advanced'),
    '#process' => array('ec_search_filter_advanced' => array()),
    'fields' => array(
      'submit' => array(
        '#type' => 'submit',
        '#value' => t('Advanced search'),
        '#prefix' => '<div class="action">',
        '#suffix' => '</div>',
        '#weight' => 99,
      ),
    ),
  );
  $form['#query'] = isset($query) ? $query : array();

  if (isset($query)) {
    $query = array_map(create_function('$a',
      'return is_array($a) ? implode(",", $a) : $a;'), $query);
    $form['results'] = array(
      '#value' => ec_search_result($search_id, $query),
      '#weight' => 99,
    );
  }
  return $form;
}

function ec_search_filter_advanced($element) {
  // If there are no fields within the advanced search then remove the
  // advanced search field.
  $children = element_children($element);
  if (count($children) > 2) {
    return;
  }
  return $element;
}

/**
 * Query the transaction (and other associated tables) in a myriad of ways.
 *
 * @param $search_id
 *   The definition of what search is to be executed.
 * @param $selection
 *   Object representing the search criteria.
 * @param $show_details
 *   Whether to show details of the search criteria.
 * @param $show_permlink
 *   Whether to show a permanent link for a search.
 * @return
 *   A sortable HTML table with the transactions matching the search criteria.
 */
function ec_search_result($search_id, $selection, $show_details = true, $show_permlink = true) {
  include_once drupal_get_path('module', 'ec_common') .'/ec_query.inc';

  $output = '';

  // If we have a valid query object, let's hit the database.
  if (!empty($selection)) {
    $query = new _ec_query();

    $function = $search_id .'_ec_search_init';
    if (function_exists($function)) {
      $function($query);
    }

    $query->is_distinct();

    $filters = ec_search_build_filter($selection);

    foreach ($filters as $filter) {
      foreach (module_implements('ec_search') as $module) {
        $f = $module .'_ec_search';
        $f($search_id, $query, $filter, $details);
      }
      $get[] = ec_search_uri($filter);
    }

    $output .= '<div class="transactions">'. ec_search_format($search_id,
      NULL, $query, $show_details ? $details :  array()) .'</div>';
    if (!empty($show_permlink)) {
      $output .= '<p>'. l('#', $_REQUEST['q'], array(
        'title' => t('Permanent link to this search query.')),
        implode('&', $get)) .'</p>';
    }
  }

  return $output;
}

function ec_search_build_filter($selection) {
  $filters = array();
  foreach ($selection as $criterion => $criterion_value) {
    $filter = array(
      'name' => $criterion,
      'data' => array(),
      'op' => NULL,
      'value' => trim($criterion_value),
    );

    // Handle any comparison operators
    if (strpos($filter['value'], ',') !== false) {
      $filter['op'] = ',';
      $filter['data']     = explode(',', $filter['value']);
      $filter['data'] = array_map('db_escape_string', $filter['data']);
    }
    elseif (strpos($filter['value'], '-') !== false) {
      $filter['op'] = '-';
      $filter['data']     = explode('-', $filter['value']);
      $filter['data'] = array_map('db_escape_string', $filter['data']);
    }
    else {
      // This order is important.
      $ops = array('>=', '<=', '>', '<', '=');
      foreach ($ops as $op) {
        $op_len = strlen($op);
        // Only catch these operators at the beginning of the value.
        if (strlen($filter['value']) >= $op_len && substr($filter['value'],
            0, $op_len) == $op) {
          $filter['op'] = $op;
          $filter['data'][0] = db_escape_string(trim(substr($filter['value'],
            $op_len)));
          break;
        }
      }
    }

    if (empty($filter['data'])) {
      $filter['op'] = '=';
      $filter['data'][] = db_escape_string($filter['value']);
    }

    $filters[$filter['name']] = $filter;
  }

  return $filters;
}

/**
 * Format and theme the output based on a query and criteria
 */
function ec_search_format($search_id, $criteria, $query, $details = array()) {
  global $pager_total_items;
  $header = array();
  $output = '';

  foreach (module_implements('ec_search_format_header') as $module) {
    $f = $module .'_ec_search_format_header';
    if (function_exists($f)) {
      $f($search_id, $criteria, $query, $header);
    }
  }

  if (!empty($header)) {
    $count = 0;
    foreach ($header as $key => $value) {
      if (!isset($value['weight'])) {
        $header[$key]['weight'] = ++$count/1000;
      }
    }
  }

  $thead = ec_search_build_row($header);
  $query->add_order(trim(preg_replace('/order by/i', '', tablesort_sql($thead))));

  $rows = array();
  $result = pager_query($query->query(), 50, $search_id, $query->query(TRUE));
  $count = 0;
  while ($data = db_fetch_array($result)) {
    $row = array(array());
    foreach (module_implements('ec_search_format_row') as $module) {
      $f = $module .'_ec_search_format_row';
      if (function_exists($f)) {
        $f($search_id, $criteria, $query, $header, $row, $data);
      }
    }

    foreach ($row as $r) {
      $rows[] = ec_search_build_row($r, $header);
    }
  }
  if (!empty($details)) {
    $numset = $pager_total_items[$search_id];
    $output .= ($numset > 0) ?
      '<h2>'. t('%result found', array('%result' => format_plural($numset,
      '1 result', '@count results'))) .'</h2>':
      '<h2>'. t('No results found') .'</h2>';
    $output .= theme('item_list', $details, t('Search criteria:'));
    $output .= "<p>". l(t('reset'), $_REQUEST['q'],
      array('title' => t('Reset search criteria.'))) ."</p>";
  }
  elseif (empty($rows)) {
    $output .= '<h2>'. t('No information found') .'</h2>';
  }
  if (!empty($rows)) {
    $output .= theme('table', $thead, $rows);
    $output .= theme('pager', NULL, 50, $search_id);
  }
  return $output;
}

function ec_search_build_row($row, $header = NULL) {
  if (!empty($header)) {
    // Now we need to copy all the weights from the header and populate any
    // missing columns
    foreach ($header as $key => $cell) {
      if (!isset($row[$key])) {
        $row[$key] = array('data' => '');
      }
      $row[$key]['weight'] = $cell['weight'];
    }

    // remove any additional headers.
    foreach (array_diff(array_keys($row), array_keys($header)) as $col) {
      unset($row[$col]);
    }
  }

  uasort($row, 'ec_sort');
  $new_row = array();
  foreach ($row as $cell) {
    unset($cell['weight']);
    if (is_array($cell['data'])) {
      $cell['data'] = drupal_render($cell['data']);
    }
    if (!empty($header) && isset($cell['colspan']) && $cell['colspan'] == 'max') {
      $cell['colspan'] = count($header) - count($new_row);
      $new_row[] = $cell;
      break;
    }
    $new_row[] = $cell;
  }

  return $new_row;
}

/**
 * Creates a http query string part for using in transactions search.
 *
 * @param $field
 *   The name of a search criterion.
 * @param $data
 *   Array representing the values of the search criterion.
 * @return
 *   String http query string part.
 */
function ec_search_uri($filter) {
  $filter['data'][0] = !in_array($filter['op'], array(',', '=')) ? $filter['op'] . $filter['data'][0] : $filter['data'][0];
  return $filter['name'] .'='. implode(',', array_map('urlencode', $filter['data']));
}

/**
 * Converts the criteria operator to a database friendly one.
 *
 * @param $data
 *   Array representing the values of the search criterion.
 * @param $operator
 *   String representing the criterion operator.
 * @param $column
 *   String representing the database column relationed to this criterion.
 * @param $callback
 *   A callback function to run in every values of $data.
 * @return
 *   A string representing a sql WHERE clause part.
 */
function ec_search_operator($data, $operator, $column, $callback = NULL) {

  $query = '';

  switch ($operator) {

    case '-':
      $data = ($callback) ? array_map($callback, $data) : $data;
      for ($i = 0; $i < 2; $i++) {
        $query .= ($i == 0) ? "({$column} >= '{$data[$i]}'" : " AND {$column} <= '{$data[$i]}'";
      }
      return $query .')';

    case ',':
      $data = ($callback) ? array_map($callback, $data) : $data;
      return "{$column}  IN ('" . implode("','", $data) ."')";

    default:
      $data[0] = ($callback) ? $callback($data[0]) : $data[0];
      return "{$column} {$operator} '{$data[0]}'";
  }

  return FALSE;
}

/**
 * Format a string for using in Search criteria details.
 *
 * @param $label
 *   Nice name of the search criterion.
 * @param $data
 *   Array representing the values of the search criterion.
 * @param $operator
 *   String representing a criterion operator.
 * @return
 *   HTML string with label, operator and value of the criterion.
 */
function ec_search_detail($label, $filter, $data = NULL) {
  $data = empty($data) ? $filter['data'] : $data;
  switch ($filter['op']) {
    case ',':
      $value = ' = '. implode(',', $data);
      break;

    case '-':
      $value = " {$data[0]} - {$data[1]}";
      break;

    default:
      $value = " {$filter['op']} ". implode(',', $data);
  }
  return '<strong>'. check_plain($label) .'</strong>'. check_plain($value);
}

