<?php

/**
 * @file product.admin.inc
 *   This file contains all the administration code for configuring
 *   e-Commerce products - features, attributes, types.
 * @author e-Commerce Dev Team
 */

// -------------------------------------------------------------------------
// Node add/edit form functions

/**
 * Elements common to every single product: price, SKU and
 * the option to show a "Add to cart" as a link or a button
 */
function product_form_base_elements(&$node) {
  $form['ptype'] = array(
    '#type' => 'value',
    '#value' => $node->ptype
  );
  $form['price'] = array(
    '#type' => 'price',
    '#title' => t('Price'),
    '#size' => 25,
    '#maxlength' => 50,
    '#required' => TRUE,
    '#default_value' => format_currency(!empty($node->price) ? $node->price : 0),
    '#description' => t('How much does this product retail for? Note: This price may be different from the selling price due to price adjustments elsewhere.'),
    '#weight' => -20,
  );
  $form['sku'] = array(
    '#type' => 'textfield',
    '#title' => t('SKU'),
    '#size' => 25,
    '#maxlength' => 50,
    '#default_value' => !empty($node->sku) ? $node->sku : '',
    '#description' => t('If you have an unique identifier for this product from another system or database, enter that here. This is optional, as system IDs are automatically created for each product.'),
    '#weight' => -10,
  );
  // Prepare an area of the form that is collapsed by default.
  // Default/rare values can be tucked away here.
  $form['product_collapsed'] = array(
    '#type' => 'fieldset',
    '#title' => t('Special options'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    'hide_cart_link' => array(
      '#type' => 'radios',
      '#title' => t('"Add to cart" link'),
      '#default_value' => isset($node->hide_cart_link) ? $node->hide_cart_link : 0,
      '#options' => array(t('Visible'), t('Hidden')),
      '#description' =>  t('Choose whether or not you want the "Add to cart" link visible for this product.'),
      '#weight' => -5,
    ),
  );
  return $form;
}

/**
 * Generates product type options fieldset.
 * Calling function (product_form_alter) has already checked that the node is a product.
 */
function _product_alter_node_form(&$node) {
  drupal_add_css(drupal_get_path('module', 'product') .'/product.css');
  // Product type collapsed fieldset
  $form = array(
    '#type' => 'fieldset',
    '#title' => t('Product'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  // Choose a product type
  $ptypes[FALSE] = array('name' => t('Not a product'), 'help' => t('This item not for sale.'));
  $types = variable_get('ec_product_ptypes_'. $node->type, NULL);
  if (empty($types)) {
    $types = array_keys(product_ptypes_get('names'));
  }

  // Populate product type properties.
  foreach ($types as $key) {
    $info = product_ptypes_get('type', $key);
    if (product_ptypes_access($info->name)) {
      $ptypes[$key] = array(
        'name' => $info->name,
        'help' => $info->description,
      );
    }
  }
  // Display each radio option, with descriptive text
  foreach ($ptypes as $key => $data) {
    $form['ptype'][$key] = array(
      '#type' => 'radio',
      '#title' => $data['name'],
      '#return_value' => $key,
      '#default_value' => $node->ptype,
      '#description' => $data['help'],
      '#parents' => array('ptype'),
    );
  }
  $form['product_add'] = array(
    '#type' => 'button',
    '#value' => t('Add to store')
  );
  return $form;
}

/**
 * Generates product form fields to be added into the node form.
 * Called from product_form_alter().
 *
 * @param $form_id, $form_values, as passed to hook_form_alter.
 * @ingroup form
 */
function _product_alter_product_form($form_id, &$node_form) {
//  dpr($node_form);
  $product_type = product_ptypes_get('type', $node_form['#node']);
  // Get base form elements.
  $pform = array(
    '#type' => 'fieldset',
    '#title' => t('Product'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t('Product details') .' ('. $product_type->name .')',
  );
  $pform = array_merge($pform, product_form_base_elements($node_form['#node']));
  $pform['ptype']['#type'] = 'hidden';

  // If this type is from a specific module, get its bits.
  // TODO: alter receiving modules so that they listen for hook_product_form_alter().
  $pform = array_merge($pform, (array) product_invoke_productapi($node_form['#node'], 'form'));

  // Allow other modules to alter the product form ($pform).
  // Preferable over them implementing hook_form_alter directly.
  foreach (module_implements('product_form_alter') as $module) {
    $func = $module .'_product_form_alter';
    $func($pform, $node_form);
  }

  // Add remove option if applicable.
  if (variable_get('ec_product_convert_'. $node_form['#node']->type, EC_PRODUCT_CONVERT_NEVER) != EC_PRODUCT_CONVERT_ALWAYS) {
    $pform['product_collapsed']['product_remove'] = array(
      '#type' => 'checkbox',
      '#title' => t('Remove from store'),
      '#description' => t('Check here to delete product information. Takes effect when changes are submitted.')
    );
  }
  return $pform;
}

/**
 * Additional settings that are added to the node type form at admin/content/types/*
 *
 * @param $form_id, $form_values, as passed to hook_form_alter.
 * @ingroup form
 */
function _product_alter_node_type_form($form_id, &$form) {
  $form['product'] = array(
    '#type' => 'fieldset',
    '#title' => t('Product'),
    '#collapsible' => TRUE,
    '#weight' => 1,
  );
  $type = $form['old_type']['#value'];
  $form['product']['ec_product_convert'] = array(
    '#type' => 'select',
    '#title' => t('Node type is always a product'),
    '#default_value' => product_can_be_converted($type),
    '#options' => array(EC_PRODUCT_CONVERT_NEVER => t('Never'), EC_PRODUCT_CONVERT_ALWAYS => t('Always'), EC_PRODUCT_CONVERT_OPTIONAL => t('Optional')),
    '#description' => t('Select the setting which will determine how the product is going to interact with the node'),
  );
  $ptypes = product_ptypes_get();
  $options = array();
  foreach ($ptypes as $ptype => $info) {
    drupal_add_css(drupal_get_path('module', 'product') .'/product.css');
    $options[$ptype] = t('!name<dt>!description</dt>',
      array('!name' => $info->name, '!description' => $info->description));
  }
  $ptypes = variable_get('ec_product_ptypes_'. $type, array_keys(product_ptypes_get('names')));
  $form['product']['ec_product_ptypes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Product types'),
    '#default_value' => $ptypes,
    '#options' => $options,
    '#description' => t('Indicate the products types that wil be available on the current type of node.'),
  );
  $form['delete']['#weight'] = 11;
  $form['submit']['#weight'] = 10;
}

// -------------------------------------------------------------------------
// Admin functions, only appearing on admin/ecsettings/ptypes

/**
 * List all product types
 * Path: admin/ecsettings/ptypes
 */
function product_admin_ptypes() {
  $ptypes = product_ptypes_get();
  uasort($ptypes, create_function('$a, $b', 'if ($a->name == $b->name) { return 0; } else { return strcmp($a->name, $b->name); }'));
  $output = '';
  if (!empty($ptypes)) {
    $head = array(t('Name'), t('Type'), t('Description'), array(
      'data' => t('Operations'), 'colspan' => 2));
    foreach ($ptypes as $ptype => $item) {
      $rows[] = array(
        array('data' => l($item->name, "admin/ecsettings/ptypes/{$ptype}")),
        array('data' => check_plain($ptype)),
        array('data' => check_plain($item->description)),
        array('data' => l(t('edit'), "admin/ecsettings/ptypes/{$ptype}")),
        array('data' => !isset($item->internal) ? l(t('delete'),
          "admin/ecsettings/ptypes/{$ptype}/delete") : ''),
      );
    }
    $output .= theme('table', $head, $rows);
  }
  else {
    drupal_set_message(t('No product types defined.'));
  }
  return $output;
}

/**
 * Add or Edit product type form.
 * Path: admin/ecsettings/ptypes/add, admin/ecsettings/ptypes/*
 */
function product_admin_ptypes_form($info = NULL) {
  $form = array();
  if (empty($info)) {
    $info = new stdClass;
    $info->name = $info->ptype = $info->description = '';
  }
  else {
    $form['old_ptype'] = array(
      '#type' => 'value',
      '#value' => $info->ptype,
    );
  }
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#default_value' => $info->name,
    '#description' => t('The human readable name of this product type.'),
    '#required' => TRUE,
  );
  $form['ptype'] = array(
    '#type' => 'textfield',
    '#title' => t('Type'),
    '#default_value' => $info->ptype,
    '#description' => t('The machine-readable name of this product type. This is used by the system internally to refer to this product type.'),
    '#required' => TRUE,
  );
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => $info->description,
    '#description' => t('A brief description of what this product type is.'),
    '#required' => TRUE,
  );
  $form['info'] = array(
    '#type' => 'value',
    '#value' => $info,
  );
  $form['submit'] = array();
  $form['submit']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Submit product type'),
  );
  if (isset($info->internal) && empty($info->internal)) {
    $form['submit']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset product type'),
    );
  }
  elseif (!empty($info->ptype) && !db_result(db_query_range("SELECT COUNT(*) FROM {ec_product}
      WHERE ptype = '%s'", $info->ptype, 0, 1))) {
    $form['submit']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete product type'),
    );
  }
  return $form;
}

/**
 * Validate product_admin_ptypes_form()
 * @ingroup form
 */
function product_admin_ptypes_form_validate($form_id, $form_values) {
  if (!empty($form_values['old_ptype']) && $form_values['ptype'] != $form_values['old_ptype'] &&
      product_ptypes_get('type', $form_values['ptype'])) {
    form_set_error('ptype', t('Product type %ptype already exists',
      array('%ptype' => $form_values['ptype'])));
  }
}

/**
 * Submit product_admin_ptypes_form()
 * @ingroup form
 */
function product_admin_ptypes_form_submit($form_id, $form_values) {
  $info = $form_values['info'];
  $info->name = $form_values['name'];
  $info->ptype = $form_values['ptype'];
  $info->old_ptype = isset($form_values['old_ptype']) ? $form_values['old_ptype'] : NULL;
  $info->description = $form_values['description'];

  if ($form_values['op'] == t('Reset product type')) {
    product_ptypes_delete($info->ptype);
    drupal_set_message(t('Product type %ptype has been reset',
      array('%ptype' => $info->ptype)));
    return 'admin/ecsettings/ptypes/'. $info->ptype;
  }
  elseif ($form_values['op'] == t('Delete product type')) {
    return 'admin/ecsettings/ptypes/'. $info->ptype .'/delete';
  }
  elseif ($form_values['op'] == t('Submit product type')) {
    product_ptypes_save($info);
  }
  return 'admin/ecsettings/ptypes';
}

/**
 * Implementation of hook_delete().
 */
function _product_delete($node, $revision_only = FALSE) {
  if (!empty($revision_only)) {
    db_query('DELETE FROM {ec_product} WHERE nid = %d AND vid = %d', $node->nid, $node->vid);
  }
  else {
    db_query('DELETE FROM {ec_product} WHERE nid = %d', $node->nid);
  }
  module_invoke('cart', 'productapi', $node, 'delete');
  product_invoke_productapi($node, 'delete', $revision_only);
  product_invoke_feature_all($node, 'delete', $revision_only);
  drupal_set_message(t('Product deleted'));
}

/**
 * @param $form_id
 * @param $form_values
 * @ingroup form
 */
function _product_edit_form(&$invoice) {
  $form['#theme'] = 'product_edit_form';
  $form['products'] = array(
    '#type' => 'fieldset',
    '#title' => t('Products'),
    '#tree' => TRUE,
    '#theme' => 'product_edit_form_product',
  );
  foreach ((array)$invoice->items as $key => $item) {
    $node = node_load($item->nid);
    $form['products'][$node->nid]['#node'] = $item;
    $form['products'][$node->nid]['title'] = array(
      '#value' => l($item->title, 'node/'. ($node->pparent ? $node->pparent : $node->nid)),
    );
    $price = store_adjust_misc($invoice, $item);
    $form['products'][$node->nid]['#total'] = product_has_quantity($node) ?
      $price * $item->qty : $price;
    if (!empty($node->is_recurring)) {
      $form['products'][$node->nid]['recurring'] = array(
        '#value' => product_recurring_nice_string($node)
      );
    }
    if ($node->ptype == 'tangible' && $node->availability != NULL && $node->availability != 1) {
      $form['products'][$node->nid]['availability'] = array(
        '#value' => availability_get_message($node->availability)
      );
    }
    $form['products'][$node->nid]['qty'] = array(
      '#type' => product_has_quantity($node) ? 'textfield' : 'value',
      '#default_value' => $item->qty,
      '#size' => 2,
      '#maxlength' => 2
    );
    $form['products'][$node->nid]['delete'] = array(
      '#type' => 'checkbox',
    );
    if ($extra = product_invoke_productapi($item, 'cart_form', $invoice)) {
      $form['#product_cart_form'] = TRUE;
      $form['products'][$node->nid]['data'] = array();
    }
  }
  $form['add'] = array(
    '#type' => 'fieldset',
    '#title' => t('Add Products'),
    '#collapsible' => TRUE,
    '#collapsed' => count($invoice->items) ? TRUE : FALSE,
  );
  $form['add']['new_products'] = array(
    '#type' => 'textfield',
    '#title' => t('Product Ids'),
    '#maxlength' => 60,
    '#autocomplete_path' => 'product/autocomplete/all',
    '#description' => t('Enter the id\'s of the products that you which to add to the invoice. Here is a <a href="!product_quicklist" onclick="window.open(this.href, \'!product_quicklist\', \'width=480,height=480,scrollbars=yes,status=yes\'); return FALSE">list of all products</a>.', array('!product_quicklist' => url('admin/store/products/quicklist'))),
  );
  $form['submit']['invoiceop'] = array(
    '#type' => 'submit',
    '#value' => t('Update Products'),
    '#attributes' => array('id' => 'invoiceop'),
  );
  $form['submit']['continue'] = array(
    '#type' => 'submit',
    '#value' => t('Continue'),
  );
  return $form;
}

/**
 * Assumes the node data has already been saved to its respective tables and
 * only updates the main product table and the plugin of the product type
 * (e.g., product_tangible).
 *
 * @param $node
 *   Object, the node object
 */
function _product_save($node) {
  if (!empty($node->ptype)) {
    $fields = product_fields();
    // Be very sure that we have a product entry to update!
    if (db_result(db_query('SELECT COUNT(vid) FROM {ec_product} WHERE vid = %d', $node->vid))) {
      foreach ($fields as $field) {
        if (isset($node->$field)) {
          $q[] = db_escape_string($field) ." = '%s'";
          $v[] = $node->$field;
        }
      }
      $v[] = $node->vid;
      db_query('UPDATE {ec_product} SET '. implode(', ', $q) .' WHERE vid = %d', $v);
      product_invoke_productapi($node, 'update');
      product_invoke_feature_all($node, 'update');
    }
    else {
      foreach ($fields as $field) {
        if (isset($node->$field)) {
          $k[] = db_escape_string($field);
          $v[] = $node->$field;
          $s[] = "'%s'";
        }
      }
      db_query('INSERT INTO {ec_product} ('. implode(', ', $k) .') VALUES('. implode(', ', $s) .')', $v);
      product_invoke_productapi($node, 'insert');
      product_invoke_feature_all($node, 'insert');
    }
  }
}

/**
 * List all features from a given product type.
 * Path: admin/ecsettings/ptypes/.../features
 *
 * @param $info
 *   Object, the product type
 */
function product_admin_ptypes_feature_list($info) {
  $form = array();
  if (!empty($info->features)) {
    $form['features'] = array(
      '#tree' => TRUE,
    );
    foreach ($info->features as $ftype => $feature) {
      $form['features'][$ftype]['ptype'] = array(
        '#type' => 'value',
        '#value' => $info->ptype,
      );
      $form['features'][$ftype]['ftype'] = array(
        '#type' => 'value',
        '#value' => $feature->ftype,
      );
      $form['features'][$ftype]['name'] = array(
        '#value' => $feature->name,
      );
      $form['features'][$ftype]['description'] = array(
        '#value' => $feature->description,
      );
      $form['features'][$ftype]['weight'] = array(
        '#type' => 'weight',
        '#default_value' => $feature->weight,
        '#disabled' => isset($feature->allow_edit) ? !$feature->allow_edit : FALSE,
      );
      if (!isset($feature->allow_disable) || $feature->allow_disable) {
        $form['features'][$ftype]['delete'] = array(
          '#value' => l(t('delete'), 'admin/ecsettings/ptypes/'. $info->ptype .
          '/features/'. $feature->ftype .'/delete'),
        );
      }
    }
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Update features',
    );
  }
  else {
    drupal_set_message(t('No features have been added to this product'));
  }
  return $form;
}

/**
 * Submit for product_admin_ptypes_feature_list()
 * @ingroup form
 */
function product_admin_ptypes_feature_list_submit($form_id, $form_values) {
  foreach ($form_values['features'] as $feature) {
    product_feature_update((object)$feature);
  }
  drupal_set_message(t('Features have been updated'));
}

/**
 * Link product types to features.
 * @ingroup form
 * Path: 'admin/ecsettings/ptypes/.../add_feature
 */
function product_admin_ptypes_feature_form($info) {
  $options = $form = array();
  $features = product_feature_get();
  foreach ($features as $ftype => $feature) {
    if (!isset($info->features[$ftype])) {
      $options[$ftype] = t('@name<dt>@description</dt>',
        array('@name' => $feature->name, '@description' => $feature->description));
    }
  }
  if (!empty($options)) {
    $form['info'] = array(
      '#type' => 'value',
      '#value' => $info,
    );
    $form['ftype'] = array(
      '#type' => 'radios',
      '#title' => t('Select a product feature to be added'),
      '#options' => $options,
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Add product feature'),
    );
  }
  else {
    drupal_set_message(t('No more features can be added to @name',
      array('@name' => $info->name)));
    drupal_goto('admin/ecsettings/ptypes/'. $info->ptype .'/features');
  }
  return $form;
}

/**
 * Submit for product_admin_ptypes_feature_form()
 * @ingroup form
 */
function product_admin_ptypes_feature_form_submit($form_id, $form_values) {
  product_feature_enable($form_values['info'], $form_values['ftype']);
  return 'admin/ecsettings/ptypes/'. $form_values['info']->ptype .'/features';
}

/**
* Overview of settings of a ptype.
* @ingroup $form
* Path: admin/ecsettings/ptypes/settings
*/
function product_admin_ptypes_settings() {
  $form = array();

  $form['ec_product_convert'] = array(
    '#type' => 'select',
    '#title' => t('New node types product conversion setting'),
    '#default_value' => variable_get('ec_product_convert', 0),
    '#options' => array(
      0 => t('Never'),
      1 => t('Always'),
      2 => t('Optional'),
    ),
    '#description' => t('This setting will be applied to all new node types when they are created'),
  );

  $form['ec_product_types'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add new product types to nodes automatically'),
    '#default_value' => variable_get('ec_product_types', 1),
    '#description' => t('Set the default action for all new product types'),
  );

  return system_settings_form($form);
}

/**
 * Delete a product type.
 * @ingroup $form
 * Path: admin/ecsettings/ptypes/.../delete
 */
function product_admin_ptypes_delete($info) {
 $form = array();

 if (db_result(db_query_range("SELECT COUNT(*) FROM {ec_product}
     WHERE ptype = '%s'", $info->ptype, 0, 1))) {
   drupal_set_message(t('You cannot delete product type %name as products still exist',
     array('%name' => $info->name)), 'error');
   drupal_goto('admin/ecsettings/ptypes');
 }

 $form['info'] = array(
   '#type' => 'value',
   '#value' => $info,
 );

 return confirm_form($form, t('Are you sure you want to delete product type %name?',
   array('%name' => $info->name)), 'admin/ecsettings/ptypes', t('This action cannot be undone'),
   t('Delete product type'), t('Cancel'));
}

/**
 * Submit product_admin_ptypes_delete()
 * @ingroup form
 */
function product_admin_ptypes_delete_submit($form_id, $form_values) {
 $info = $form_values['info'];
 product_ptypes_delete($info->ptype);
 drupal_set_message(t('Product type %name has been deleted', array('%name' => $info->name)));
 return 'admin/ecsettings/ptypes';
}

/**
 * Delete a product type
 */
function product_ptypes_delete($ptype) {
  $info = product_ptypes_get('type', $ptype, TRUE);
  db_query("DELETE FROM {ec_product_ptypes} WHERE ptype = '%s'", $ptype);
  module_invoke_all('product_types', $info, 'delete');
}

/**
 * Save a product type
 */
function product_ptypes_save($info) {
  $ptype = isset($info->old_ptype) ? $info->old_ptype : $info->ptype;
  if (db_result(db_query_range("SELECT COUNT(*) FROM {ec_product_ptypes}
      WHERE ptype = '%s'", $ptype, 0, 1))) {
    db_query("UPDATE {ec_product_ptypes} SET name = '%s', ptype = '%s',
      description = '%s' WHERE ptype = '%s'", $info->name, $info->ptype,
      $info->description, $ptype);
  }
  else {
    db_query("INSERT INTO {ec_product_ptypes} (name, ptype, description)
      VALUES ('%s', '%s', '%s')", $info->name, $info->ptype, $info->description);
  }
  $info = product_ptypes_get('type', $info->ptype, TRUE);
  module_invoke_all('product_types', $info, 'update');
}

/**
 * Builds a list of available node types, by quering hook_product_info() in
 * all modules.
 *
 * Note: this is called by product_ptypes_get(), which caches the values
 * and hence this is kept in the admin file.
 */
function product_ptypes_build() {
  $ptypes = array();
  $names = array();

  $info_array = module_invoke_all('product_info');
  foreach ($info_array as $type => $info) {
    $ptypes[$type] = (object)$info;
    $ptypes[$type]->ptype = $type;
    $ptypes[$type]->internal = TRUE;
    $ptypes[$type]->features = array();
    $names[$type] = $info['name'];
  }

  $result = db_query('select * FROM {ec_product_ptypes}');
  while ($type = db_fetch_object($result)) {
    if (!empty($ptypes[$type->ptype])) {
      if ($type->name) {
        $ptypes[$type->ptype]->name = $type->name;
      }
      if ($type->description) {
        $ptypes[$type->ptype]->description = $type->description;
      }
      $ptypes[$type->ptype]->internal = FALSE;
      $names[$type->ptype] = $ptypes[$type->ptype]->name;
    }
    else {
      $type->module = 'product_generic';
      $type->features = array();
      $ptypes[$type->ptype] = $type;
      $names[$type->ptype] = $type->name;
    }
  }

  $features = product_feature_get();
  $result = db_query('SELECT * FROM {ec_product_features} ORDER BY weight ASC');
  while ($data = db_fetch_object($result)) {
    if (isset($ptypes[$data->ptype])) {
      if (isset($features[$data->ftype])) {
        $feature = $features[$data->ftype];
        $feature->weight = $data->weight;
        $ptypes[$data->ptype]->features[$feature->ftype] = $feature;
      }
    }
  }

  asort($names);
  return array($ptypes, $names);
}

/**
 * Implementation of hook_product_types()
 * Which is also in the admin file.
 */
function product_product_types($info, $op) {
  switch ($op) {
    case 'update':
      if (!is_null($info->old_ptype) && variable_get('ec_product_ptypes', 1)) {
        foreach (node_get_types('names') as $type => $description) {
          $ptypes = variable_get('ec_product_ptypes_'. $type, array());
          if (!in_array($info->ptype, $ptypes)) {
            $ptypes[] = $info->ptype;
          }
        }
      }
      break;

    case 'delete':
      foreach ($info->features as $ftype => $feature) {
        product_feature_disable($info, $ftype);
      }
  }
}


/**
 * Confirm if the product feature should be deleted
 *
 * @param $info
 * @param $features
 *   Object, the feature
 * @ingroup form
 */
function product_admin_ptypes_feature_delete($info, $feature) {
  $form = array();

  $form['info'] = array(
    '#type' => 'value',
    '#value' => $info,
  );
  $form['feature'] = array(
    '#type' => 'value',
    '#value' => $feature,
  );

  return confirm_form($form, t('Are you sure you want to delete feature %feature_name from product type %name?',
    array('%feature_name' => $feature->name, '%name' => $info->name)),
    'admin/ecsettings/ptypes', t('This action cannot be undone'), t('Delete feature'),
    t('Cancel'));
}

/**
 * Confirm if the product feature should be deleted
 *
 * @param $info
 * @param $features
 *   Object, the feature
 * @ingroup form
 */
function product_admin_ptypes_feature_delete_submit($form_id, $form_values) {
  product_feature_disable($form_values['info'], $form_values['feature']->ftype);
  return 'admin/ecsettings/ptypes/'. $form_values['info']->ptype .'/features';
}

// -------------------------------------------------------------------------
// Theme functions

/**
 * @ingroup themeable
 */
function theme_product_admin_ptypes_feature_list($form) {
  $output = '';

  if (isset($form['features'])) {
    $heading = array(t('Name'), t('Description'), t('Weight'), t('Operations'));
    foreach (element_children($form['features']) as $element) {
      $rows[] = array(
        array('data' => drupal_render($form['features'][$element]['name'])),
        array('data' => drupal_render($form['features'][$element]['description'])),
        array('data' => drupal_render($form['features'][$element]['weight'])),
        array('data' => isset($form['features'][$element]['delete']) ?
          drupal_render($form['features'][$element]['delete']) : ''),
      );
    }

    $output .= theme('table', $heading, $rows);
  }

  return $output . drupal_render($form);
}

/**
 * @ingroup themeable
 */
function theme_product_edit_form(&$form) {
  $output .= drupal_render($form['products']);
  $output .= drupal_render($form);
  return $output;
}

/**
 * @ingroup themeable
 */
function theme_product_edit_form_product(&$form) {
  $children = element_children($form);
  if (!empty($children)) {
    $extra = FALSE;

    foreach ($children as $nid) {
      if (!empty($form[$nid]['data'])) {
        $extra = TRUE;
        break;
      }
    }

    foreach ($children as $nid) {
      $total+= $form[$nid]['#total'];
      $desc = drupal_render($form[$nid]['title']) .'<br />';
      if (!empty($form[$nid]['recurring'])) {
        $desc .= '<div class="recurring-details">'. drupal_render($form[$nid]['recurring']) .'</div>';
      }
      if (!empty($form[$nid]['availability'])) {
        $desc .= drupal_render($form[$nid]['availability']);
      }
      $desc .= '<p>'. format_currency($form[$nid]['#total']) .'</p>';
      $row = array(
        array('data' => $desc),
        array('data' => drupal_render($form[$nid]['qty'])),
      );
      if (!empty($extra)) {
        if (!empty($form[$nid]['node'])) {
          $row[] = array('data' => drupal_render($form[$nid]['node']));
          $extra = TRUE;
        }
        else {
          $row[] = '';
        }
      }
      $row[] = array('data' => drupal_render($form[$nid]['delete']), 'align' => 'center');
      $rows[] = $row;
    }
    $header = array(
      array('data' => t('Items')),
      array('data' => t('Qty')),
    );
    if (!empty($extra)) {
      $header[] = array('data' => '');
    }
    $header[] = array('data' => t('Delete'));
    $output .= theme('table', $header, $rows);
  }
  else {
    $output .= '<p>'. t('No Products Currently on this Invoice') .'</p>';
  }
  $output .= drupal_render($form);
  return $output;
}

/**
 * Display all products in a table format.
 * @ingroup themable
 */
function theme_product_overview($rows, $header) {
  if (!empty($rows)) {
    $output = theme('table', $header, $rows);
  }
  else {
    $output = '<p>'. t('There are no products to view.') .'</p>';
  }
  return $output;
}
