<?php

/**
 * @file product.views.inc
 *   This file provides product integration with the views module.
 * @author e-Commerce Dev Team
 */

// -------------------------------------------------------------------------
// Views hooks

/**
 * Implementation of hook_views_tables()
 */
function product_views_tables() {
  $tables['ec_product'] = array(
    'name' => 'ec_product',
    'join' => array(
      'left' => array(
        'table'     => 'node',
        'field'     => 'vid'
      ),
      'right' => array(
        'field'     => 'vid'
      )
    ),
    'fields' => array(
      'sku' => array(
        'name'      => t('Product: SKU'),
        'sortable'  => true,
      ),
      'price' => array(
        'handler' => 'product_views_handler_price',
        'name' => t('Product: Price'),
        'sortable'  => false,
      ),
    ),
    'sorts' => array(
      'sku' => array(
        'name' => t('Product: SKU'),
      ),
      'price' => array(
        'name' => t('Product: Price'),
      )
    ),
    'filters' => array(
      'ptype' => array(
        'help'      => 'Filter by product type',
        'name'      => 'Product: Type',
        'operator'  => 'views_handler_operator_andor',
        'value'     => array(
          '#multiple' => true,
          '#options'  => product_ptypes_get('names'),
          '#type'     => 'select'
        ),
        'value-type' => 'array'
      ),
      'is_product' => array(
        'name' => 'Product: Is Product',
        'operator' => array('=' => 'Equals'),
        'list' => 'views_handler_operator_yesno',
        'list-type' => 'select',
        'handler' => 'product_views_handler_filter_product',
      ),
    )
  );
  return $tables;
}

/**
 * Set a new listing style tailored for products
 * @return
 *   Array, new listing styles
 */
function product_views_style_plugins() {
  $items['product_list'] = array(
    'name' => t('Product: List of products'),
    'theme' => 'product_views_product_list',
    'summary_theme' => 'views_summary',
  );
  return $items;
}

// -------------------------------------------------------------------------
// Product views callbacks

/**
 * Filter Nodes based on being a product.
 */
function product_views_handler_filter_product($op, $filter, $filterinfo, &$query) {
  switch ($op) {
    case 'handler':
      $query->ensure_table('ec_product');
      switch ($filter['value'][0]) {
        case '0':
          $query->add_where('ec_product.ptype IS NULL');
          break;

        case '1':
          $query->add_where('ec_product.ptype IS NOT NULL');
          break;
      }
      break;
  }
}

/**
 * Field handler to format prices in the correct format.
 */
function product_views_handler_price($fieldinfo, $fielddata, $value, $data) {
  $node = node_load($data->nid);
  return format_currency(product_price_adjust($node, 'product') + product_specials_get($node, 'product', TRUE));
}

/**
 * Themes a product page into a table similar to that in the previous 4.7 product page.
 * @ingroup themeable
 */
function theme_product_views_product_list($view, $nodes, $type) {
  $teasers = true;
  $links = true;
  $rows = $row = array();
  foreach ($nodes as $count => $n) {
    $node = node_load($n->nid);
    if ($count > 0 and !($count % 3)) {
      $rows[] = $row;
      $row = array();
    }
    array_push($row, node_view($node, $teasers, false, $links));
  }
  if (!empty($row)) {
    array_push($rows, $row);
  }
  drupal_add_css(drupal_get_path('module', 'cart') .'/cart.css');
  return theme('table', array(), $rows, array('class' => 'product-table'));
}

// -------------------------------------------------------------------------
// Default views

/**
 * Set a standard views. The /products page will show a list
 * of available products, similar to old hardcoded products page.
 */
function product_views_default_views() {
  $view = new stdClass();
  $view->name               = 'products';
  $view->description        = 'Product listing';
  $view->page_title         = 'Products';
  $view->page_type          = 'product_list';
  $view->url                = 'product';
  $view->use_pager          = true;
  $view->nodes_per_page     = '9';
  $view->menu               = true;
  $view->menu_title         = 'Products';
  $view->field              = array(
    array(
      'tablename' => 'node',
      'field' => 'title',
      'label' => '',
      'handler' => 'views_handler_field_nodelink',
      'options' => 'link',
    ),
    array(
      'tablename' => 'node',
      'field' => 'body',
      'label' => '',
      'handler' => 'views_handler_field_teaser',
    )
  );
  $view->filter             = array(
    array(
      'tablename' => 'node',
      'field' => 'status',
      'operator' => '=',
      'options' => '',
      'value' => '1',
    ),
    array(
      'tablename' => 'node',
      'field' => 'type',
      'operator' => 'OR',
      'options' => '',
      'value' => array('product'),
    )
  );
  $view->exposed_filter     = array();
  $view->requires           = array(node, ec_product);
  $views[$view->name] = $view;
  return $views;
}

// -------------------------------------------------------------------------
// Views candidates

/**
 * Display all products in a table format.
 * TODO: can this be done with views? Probably need appropriate 'Operations' field...
 */
function _product_overview() {
  $header = array(
    array('data' => t('ID'), 'field' => 'p.nid', 'sort' => 'asc'),
    array('data' => t('SKU'), 'field' => 'p.sku'),
    array('data' => t('Title'), 'field' => 'n.title'),
    array('data' => t('Price'), 'field' => 'p.price'),
    array('data' => t('Type'), 'field' => 'p.ptype'),
    array('data' => t('Operations'))
  );
  $sql = 'SELECT n.nid, n.title, p.* FROM {node} n
    INNER JOIN {ec_product} p ON n.vid = p.vid
    WHERE n.status = 1'. tablesort_sql($header);
  $result = pager_query(db_rewrite_sql($sql), 50);
  while ($data = db_fetch_object($result)) {
    $rows[] = array(
      $data->nid,
      empty($data->sku) ? '' : $data->sku,
      l($data->title, "node/$data->nid"),
      format_currency(product_price_adjust($data, 'product') +
      product_specials_get($data, 'product', TRUE)),
      $data->ptype,
      l(t('Edit'), "node/$data->nid/edit")
    );
  }
  $pager = theme('pager', NULL, 50, 0);
  if (!empty($pager)) {
    $rows[] = array(array('data' => $pager, 'colspan' => 6));
  }
  return theme('product_overview', $rows, $header);
}

/**
 * Generate a hardcoded product list. Uses echo to
 * directly print the result.
 * TODO: can this be done with views? At least this use the product list
 * above and print raw to the page.
 */
function _product_quicklist() {
  $html = store_quicklist_header();
  $header = array(
    array('data' => t('Id'), 'field' => 'p.nid', 'sort' => 'asc'),
    array('data' => t('SKU'), 'field' => 'p.sku'),
    array('data' => t('Title'), 'field' => 'n.title'),
    array('data' => t('Price'), 'field' => 'p.price'),
    array('data' => t('Type'), 'field' => 'p.ptype')
  );
  $sql = 'SELECT * FROM {node} n,
    INNER JOIN {ec_product} p ON n.vid = p.vid
    WHERE n.status = 1'. tablesort_sql($header);
  $result = pager_query(db_rewrite_sql($sql), 50);
  while ($data = db_fetch_object($result)) {
    $rows[] = array($data->nid, $data->sku, $data->title,
      format_currency(product_price_adjust($data, 'product') +
      product_specials_get($data, 'product', TRUE)), $data->ptype);
  }
  $pager = theme('pager', NULL, 50, 0);
  if (!empty($pager)) {
    $rows[] = array(array('data' => $pager, 'colspan' => 5));
  }
  echo $html;
  if (!empty($rows)) {
    echo theme('table', $header, $rows);
  }
  else {
    echo '<p>'. t('There are no products to view. You can <a href="!add_product">add a product</a>.',
      array('!add_product' => 'node/add/product')) .'</p>';
  }
  echo '</body></html>';
}
