<?php

/**
 * @file
 * Functions to communicate with shipping module api functions
 *
 * Provides integration with Australia Posts Online Postage calculator.
 * Requires the ecommerce suite of modules
 * and shpping.module installed.
 *
 * this works with drupal 4.7
 *
 * NOTES;
 *
 * Upload australiapost.inc file into your modules/ecommerce/shipping/apis folder.
 *
 * Adjust countries and shipping charges per zone at admin -> store -> settings -> shipping -> international
 *
 * Make this shipping method available to products by configuring at admin -> settings -> content types
 *
 * Once enabled for content types (products) a new form will appear when editing
 * products that allows you to include this shipping method.
 *
 * @todo ensure that the weight is in kilograms
 * @todo ensure that the lenghts is in centimeters
 */

define('AUSTRALIAPOST_URL', 'http://drc.edeliver.com.au/ratecalc.asp');
define('AUSTRALIAPOST_MAX_KGS', 20);
define('AUSTRALIAPOST_MAX_CMS', 105);
define('AUSTRALIAPOST_MIN_GIRTH_CMS', 16);

/**
 * shipping_api _get_rates_form hook.
 *
 * Request rates from Australia Post for the current transaction.  Return a form of all
 * shipping options that the shipping module will display during the checkout
 * process.
 *
 * The shipping option form only comfortably displays a simple set of shippig
 * options. So first simplicity, the current procedure is to only offer a
 * shipping method when all items can be delivered this way.
 *
 * Additionally, the current process will price each item separately or together
 * based on a global option. Grouping items together will currently simply sum
 * the volume of each item. It is the store owners responsibility
 * to determine whether the shipping price will be more accurate with all products
 * packaged together, or all separate.
 *
 */
function australiapost_get_rates(&$txn) {
  include_once drupal_get_path('module', 'store') .'/store.localization.inc';

  // Iterate through the items in the shipment, combine them into packages if appropriate.
  // If any items fall out of the norm, return and set an error.

  // Make a copy of the transaction items and pass over it making some adjustments
  // and calculations and doing some checks. Mainly we are determining valid
  // shipping methods.
  $txn_items = $txn->items;
  $standard_methods = australiapost_shipping_methods('domestic');  //TODO International, not much to do
  $standard_methods = $standard_methods['australiapost'];

  foreach ($txn_items AS $item) {
    if (!product_is_shippable($item->vid)) {
      // Ignore non-shippable products.
      unset($txn_items[$item->nid]);
    }
    else {
      // Keep count of the Australia Post shipping methods applicable to each item.
      foreach ($item->shipping_methods['australiapost'] as $item_method) {

        if (array_key_exists($item_method, $standard_methods)) {
          $standard_methods[$item_method]['count']++;
        }
      }
    }
  }

  // If there are no shippable items after the above, then return empty.
  if (!count($txn_items)) {
    return;
  }
  foreach (element_children($standard_methods) AS $method) {
    // Only keep methods that are valid for ALL items.
    if ($standard_methods[$method]['count'] < count($txn_items[0]) || !isset($standard_methods[$method]['count'])) {
      unset($standard_methods[$method]);
    }
  }

  // If we don't have any methods left, then we can't offer shipping.
  if (!count($standard_methods)) {
    return;
  }

  // We now have an array of shippable items and an array of shipping
  // methods and each item is valid for each shipping method and vice-versa.
  // Hence all items will have weight/length/width/height

//  print "X". _store_location_measure_convert(55, 'LB', 'KG'); die();




  // Calculate the total weight, volume and total volume

  // First, convert all measurements to a format expected by Australia Post.
  foreach ($txn_items as $nid => $item) {
    // Store Australia Post adjusted information here:
    $ship = array();
    $ship['qty'] = $item->qty;
  
    $attr = $item->product_attributes;
    // Weight
    $ship['weight'] = _store_location_measure_convert($attr['weight'], variable_get('ec_measure_weight', 'LB'), 'G');
    // Dimensions. Australia Post API needs millimetres, but we'd prefer to work in CM, we'll convert later.
    $unit = variable_get('ec_measure_length', 'IN');
    $ship['length'] = _store_location_measure_convert($attr['length'], $unit, 'CM');
    $ship['width'] = _store_location_measure_convert($attr['width'], $unit, 'CM');
    $ship['height'] = _store_location_measure_convert($attr['height'],$unit, 'CM');

    // Derive volume
    $item->product_attributes['volume'] = $attr['length'] * $attr['width'] * $attr['height'];
    $ship['volume'] = $ship['length'] * $ship['width'] * $ship['height'];

    $txn_items[$nid]->ship = $ship;
  }

  // Now create packages...

  // It is difficult to anticipate when maximum width/length/height is exceeded.
  // Set your maximum volume here, and we make a new package when it's exceeded.
  // This maximum will differ for each business.
  // (Australia Post use cm).
  $max_volume = variable_get('shipping_api_australiapost_maxvolume', 15000); // <-- cubic centimetres.
  $max_weight = AUSTRALIAPOST_MAX_KGS; // <-- 20kg

  // Cycle through the items, adding to packages until one of the maximum
  // values is reached for the Australia Post API. Then open a new box.

  $p = 0;
  $packages[$p] = array('volume' => 0, 'weight' => 0, 'items' => 0, 'contents' => array());
  foreach ($txn_items as $nid => $item) {
    while ($item->ship['qty'] > 0) {
      // Check if the package is getting too full (note: each package can take at least 1 item).
      $too_large = $packages[$p]['volume'] + $item->ship['volume'] > $max_volume;
      $too_heavy = $packages[$p]['weight'] + $item->ship['weight'] > $max_weight;
      if ($packages[$p]['items'] > 0 && ($too_large || $too_heavy)) {
        // Close the box.
        $packages[$p] = _australiapost_close_box($packages[$p], $item);
        // New box.
        $p++;
        $packages[$p] = array('volume' => 0, 'weight' => 0, 'items' => 0, 'contents' => array());
      }
      else {
        // Chuck item in box.
        $packages[$p]['volume'] += $item->ship['volume'];
        $packages[$p]['weight'] += $item->ship['weight'];
        $packages[$p]['items']++;
        $packages[$p]['contents'][$nid]++;
        // Calculate package dimensions.
        $item->ship['qty']--;
      }
    }
  }
  if ($packages[$p]['items'] < 1) {
    // Oops, started a new box for nothing...
    unset($packages[$p]);
  }
  else {
    // Close box.
    $packages[$p] = _australiapost_close_box($packages[$p], $item);
  }

  // With the packages, pass each to Australia Post for calculation.
  $url = variable_get('shipping_api_australiapost_url', AUSTRALIAPOST_URL);
  foreach (element_children($standard_methods) AS $method) {
    $this_rate = array(
      '#service' => 'australiapost',
      '#key' => $method,
      '#cost' => 0,
      '#hours' => variable_get('shipping_api_australiapost_turnaround', 24),
      '#currency' => 'AUD',
      '#method' => $standard_methods[$method]['#title'],
    );
    foreach ($packages as $p => $box) {
      $opts = array(
        'Pickup_Postcode' => variable_get('shipping_api_australiapost_postalcode', ''),
        'Destination_Postcode' => $txn->address['shipping']->zip,
        'Country' => strtoupper($txn->address['shipping']->country),
        'Weight' => round($box['weight']),
        'Service_Type' => $method,
        // Convert CM to MM here.
        'Length' => round($box['length'] * 10),
        'Width' => round($box['width'] * 10),
        'Height' => round($box['height'] * 10),
        'Quantity' => 1,
      );

      $ap_return = drupal_http_request(url($url, drupal_query_string_encode($opts)));
      parse_str(str_replace("\r\n", '&', $ap_return->data), $charge_data);
      if (trim($charge_data['err_msg']) == 'OK') {
        $this_rate['#cost'] += $charge_data['charge'];
        $this_rate['#hours'] = max($this_rate['#hours'], ($charge_data['days'] * 24) + $this_rate['#hours']);
        
        $error = 0; // No error
      }
      else {
        drupal_set_message(t('%description (code %code)', array('%code' => $ap_return->code, '%description' => $charge_data['err_msg'])), 'error');
        $error = -1; // negative charges indicates an error
      }
    }
  
    if (empty($error)) {
      $rates[] = $this_rate;
    }
  }

  return !empty($rates) ? $rates : -1;
}

/**
 * Helper function for australiapost_get_rates()
 */
function _australiapost_close_box($box, $last_item) {
  // For more than 1 item in a package, build in extra space to accommodate air/packaging.
  $fudge_factor = variable_get('shipping_api_australiapost_fudge', 10); // <-- percentage.

  if ($box['items'] > 1) {
    // Extra volume for multiple items in box.
    $box['volume'] += $box['volume'] * $fudge / 100;
    // Guestimate dimensions as a cube, most charge is by weight so this shouldn't effect to much.
    $box['length'] = $box['height'] = $box['width'] = round(pow($box['volume'], 1 / 3), 2);
  }
  else {
    // One item, use it's dimensions for the box.
    $box['length'] = $last_item->ship['length'];
    $box['width'] = $last_item->ship['width'];
    $box['height'] = $last_item->ship['height'];
  }
  return $box;
}

/**
 * shipping_api _product_attributes hook.
 *
 * Update the product form with fields that we need.  It is possible for
 * multiple carriers to define the same field -- that is fine.  So long as
 * the field as the same name (e.g. 'weight'), it will only be displayed
 * once, and the data will be saved and restored for use by all shipping
 * partners that define it.
 */
function australiapost_product_attributes($form) {
  include_once drupal_get_path('module', 'shipping') .'/shipping.dimensions.inc';
  $fields = array('weight', 'length', 'height', 'width');
  return _shipping_dimensions_form($form, $fields);
}

/**
 * Shipping _settings_form hook.
 */
function australiapost_settings_form(&$form) {
  $form['australiapost'] = array(
    '#type' => 'fieldset',
    '#title' => t('Australia Post settings')
  );

  $form['australiapost']['australiapost_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Australia Post server/page URL'),
    '#description' => t('Enter the fully qualified URL of the Australia Post shipping rate server, as provided by Australia Post API docs.'),
    '#default_value' => variable_get('shipping_api_australiapost_url', AUSTRALIAPOST_URL),
    '#required' => true,
  );
  $form['australiapost']['australiapost_postalcode'] = array(
    '#type' => 'textfield',
    '#title' => t('Post Code'),
    '#size' => 10,
    '#description' => t('This is the Post Code of your business and is used for calculating shipping costs.'),
    '#default_value' => variable_get('shipping_api_australiapost_postalcode', ''),
    '#required' => true,
  );
  $form['australiapost']['australiapost_turnaround'] = array(
    '#type' => 'textfield',
    '#title' => t('Turn Around Time'),
    '#size' => 10,
    '#field_suffix' => 'hours',
    '#description' => t('This is the turn around time, in hours, between receiving the order and shipping the product. It will be added to the number of hours estimated by Australia Post.'),
    '#default_value' => variable_get('shipping_api_australiapost_turnaround', 24),
    '#required' => true,
  );
  $form['australiapost']['australiapost_maxvolume'] = array(
    '#type' => 'textfield',
    '#title' => t('Max package volume'),
    '#size' => 10,
    '#field_suffix' => 'cm3',
    '#description' => t('E-commerce will try to combine multiple items before sending to Australia Post. This value, in cubic centimetres, limits the size of each package.'),
    '#default_value' => variable_get('shipping_api_australiapost_maxvolume', 15000),
    '#required' => true,
  );
  $form['australiapost']['australiapost_fudge'] = array(
    '#type' => 'textfield',
    '#title' => t('Percentage volume added for combined items.'),
    '#field_suffix' => '%',
    '#size' => 10,
    '#description' => t('If E-Commerce combines items in a box before sending to Australia Post, the outcome might be a awkward package with lots of air. This value, as a percentage, will be added to the volume of a mixed box.'),
    '#default_value' => variable_get('shipping_api_australiapost_fudge', 10),
    '#required' => false,
  );
  $form['australiapost']['australiapost_regprice'] = array(
    '#type' => 'textfield',
    '#title' => t('Domestic Registered Post Cost'),
    '#size' => 10,
    '#description' => t('Enter the current cost of registering a domestic parcel.'),
    '#default_value' => variable_get('shipping_api_australiapost_regprice', 0),
    '#required' => false,
  );

}

/**
 * shipping_api _settings_form_submit hook.
 *
 * Save data from our Australia Post-specific configuration form.
 */
function australiapost_settings_form_submit(&$form) {
  global $form_values;
  $op = $form_values['op'];

  if ($form_values['shipping_partner'] == 'australiapost') {
    variable_set('shipping_api_australiapost_postalcode',
      $form_values['australiapost_postalcode']);
    variable_set('shipping_api_australiapost_turnaround',
      $form_values['australiapost_turnaround']);
    variable_set('shipping_api_australiapost_regprice',
      $form_values['australiapost_regprice']);
  }
}

/**
 * _shipping_methods hook.
 *
 * Define the international shipping methods.
 */
function australiapost_shipping_methods($type = 'all') {

  if (empty($type)) {
    $type = 'all';
  };
  $methods = array();

  $methods['australiapost'] = array(
    '#title' => t('Australia Post'),
    '#description' => t('Australia Post'),
    '#product_attributes' => array('weight', 'length', 'width', 'width'),
  );

  if ($type == 'domestic' || $type == 'all') {
    $methods['australiapost']['STANDARD'] = array(
      '#title' => t('Standard Domestic Delivery'),
    );
    $methods['australiapost']['EXPRESS'] = array(
      '#title' => t('Express Post Domestic Delivery'),
    );
  }
  if ($type == 'international' || $type == 'all') {
    $methods['australiapost']['AIR'] = array(
      '#title' => t('International Air Delivery'),
    );
    $methods['australiapost']['SEA'] = array(
      '#title' => t('International Surface Delivery'),
    );
    $methods['australiapost']['ECI_D'] = array(
      '#title' => t('Express Courier International Document'),
    );
    $methods['australiapost']['ECI_M'] = array(
      '#title' => t('Express Courier International Merchandise'),
    );
    $methods['australiapost']['EPI'] = array(
      '#title' => t('Express Post International'),
    );
  }
  return $methods;
}
